@@ -6,10 +6,9 @@ Please find a description of this package at:
 
     http://pypi.python.org/pypi/bitarray/
 
-Thank you for using bitarray.
-- Ilan Schnell
+Author: Ilan Schnell
 """
-__version__ = '0.2.3'
+__version__ = '0.2.4'
 
 from _bitarray import _bitarray, bits2bytes
 
@@ -127,9 +126,6 @@ fromfile, tostring, fromstring
         return self
 
     # ----------------------------------------------------------
-    def __hash__(self):
-        return hash(self.tostring())
-
  
     def fromlist(self, lst):
         """fromlist(list)
@@ -151,26 +147,9 @@ This method is **deprecated**, use the extend method instead."""
         self.extend(s)
 
 
-    def _decode(self, codedict):
-        "decode\n\nXXX Not implemented yet"
-        pos = 0
-        
-        res = []
-        while pos < length:
-            res.append(sym)
-            pos += l
-        
-        print 'decode:', len(self), pos
-        if pos != length:
-            raise "ERROR"
-        
-        return res
-
-
 def test(verbosity=1):
     """test(verbosity=1)
 
 Run selftest."""
     import test_bitarray
     return test_bitarray.run(verbosity=verbosity)
-
