@@ -161,7 +161,7 @@ newbitarrayobject(PyTypeObject *type, idx_t nbits, int endian)
 }
 
 static void
-dealloc(bitarrayobject *self)
+bitarray_dealloc(bitarrayobject *self)
 {
     if (self->weakreflist != NULL)
         PyObject_ClearWeakRefs((PyObject *) self);
@@ -246,7 +246,7 @@ setrange(bitarrayobject *self, idx_t start, idx_t stop, int val)
         setbit(self, i, val);
 }
 
-/* Set ususet bits to 0, without changing slef->nbits, return bits set */
+/* Set ususet bits to 0, without changing self->nbits, return bits set */
 static int
 setunused(bitarrayobject *self)
 {
@@ -332,7 +332,7 @@ findfirst(bitarrayobject *self, int vi)
     if (self->ob_size == 0)
         return -1;
     
-    c = vi ? 0x00 : 0xff;  /* inverse! */
+    c = vi ? 0x00 : 0xff;  /* inverse */
     
     /* skip ahead by checking whole bytes */
     for (j = 0; j < self->ob_size; j++)
@@ -504,7 +504,7 @@ extend_string(bitarrayobject *self, PyObject *string, int kind)
 {
     Py_ssize_t addbits, i;
     char c, buff[256], *str, *item;
-    int bit;
+    int vi;
 
     assert (PyString_Check(string));
     addbits = PyString_Size(string);
@@ -527,8 +527,8 @@ extend_string(bitarrayobject *self, PyObject *string, int kind)
         switch (kind) {
         case STR_01:
             switch (c) {
-            case '0': bit = 0; break;
-            case '1': bit = 1; break;
+            case '0': vi = 0; break;
+            case '1': vi = 1; break;
             default:
                 sprintf(buff, "character must be '0' or '1', found '%c'", c);
                 PyErr_SetString(PyExc_ValueError, strdup(buff));
@@ -536,10 +536,10 @@ extend_string(bitarrayobject *self, PyObject *string, int kind)
             }
             break;
         case STR_RAW:
-            bit = (c == '\0') ? 0 : 1;
+            vi = c ? 1 : 0;
             break;
         }
-        setbit(self, self->nbits - addbits + i, bit);
+        setbit(self, self->nbits - addbits + i, vi);
     }
     return 0;
 }
@@ -636,8 +636,7 @@ slice_GetIndicesEx(PySliceObject *r, idx_t length,
 	if (!getIndex(r->start, start)) return -1;
 	if (*start < 0) *start += length;
 	if (*start < 0) *start = (*step < 0) ? -1 : 0;
-	if (*start >= length) 
-	    *start = (*step < 0) ? length - 1 : length;
+	if (*start >= length) *start = (*step < 0) ? length - 1 : length;
     }
     
     if (r->stop == Py_None) {
@@ -699,8 +698,7 @@ bitarray_copy(bitarrayobject *self)
     if (res == NULL)
         return NULL;
     
-    memcpy(((bitarrayobject *) res)->ob_item,
-           self->ob_item, self->ob_size);
+    memcpy(((bitarrayobject *) res)->ob_item, self->ob_item, self->ob_size);
     return res;
 }
 
@@ -721,7 +719,7 @@ bitarray_count(bitarrayobject *self, PyObject *v)
         return NULL;
     
     n1 = count(self);
-    return PyLong_FromLongLong(vi ? n1 : self->nbits - n1);
+    return PyLong_FromLongLong(vi ? n1 : (self->nbits - n1));
 }
 
 PyDoc_STRVAR(count_doc,
@@ -832,18 +830,12 @@ upon initialization.");
 static PyObject *
 bitarray_buffer_info(bitarrayobject *self)
 {
-    PyObject* res;
-    
-    res = PyTuple_New(5);
-    if (res == NULL)
-        return NULL;
-    
-    PyTuple_SET_ITEM(res, 0, PyLong_FromVoidPtr(self->ob_item));
-    PyTuple_SET_ITEM(res, 1, PyLong_FromLongLong(self->ob_size));
-    PyTuple_SET_ITEM(res, 2, PyString_FromString(ENDIANSTR(self->endian)));
-    PyTuple_SET_ITEM(res, 3, PyInt_FromLong(BITS(self->ob_size) - self->nbits));
-    PyTuple_SET_ITEM(res, 4, PyLong_FromLongLong(self->allocated));
-    return res;
+    return Py_BuildValue("OLsiL",
+			 PyLong_FromVoidPtr(self->ob_item),
+			 (idx_t) self->ob_size,
+			 ENDIANSTR(self->endian),
+			 (int) (BITS(self->ob_size) - self->nbits),
+			 (idx_t) self->allocated);
 }
 
 PyDoc_STRVAR(buffer_info_doc,
@@ -1018,7 +1010,6 @@ bitarray_bitwise(bitarrayobject *self, PyObject *args)
 {
     PyObject *other;
     char *operation;
-    enum { AND, OR, XOR } op_type;
     Py_ssize_t i;
     bitarrayobject *s, *o;
     
@@ -1030,36 +1021,36 @@ bitarray_bitwise(bitarrayobject *self, PyObject *args)
 			"bitarray objects expected for bitwise operation");
 	return NULL;
     }
-    if (strcmp(operation, "and") == 0) { op_type = AND; }
-    else if (strcmp(operation, "or") == 0) { op_type = OR; }
-    else if (strcmp(operation, "xor") == 0) { op_type = XOR; }
-    else {
-	PyErr_SetString(PyExc_ValueError, "bitwise operation unknown");
-	return NULL;
-    }
+    
     s = (bitarrayobject *) self;
     o = (bitarrayobject *) other;
-    
     if (s->nbits != o->nbits) {
 	PyErr_SetString(PyExc_ValueError,
 			"bitarrays of equal length expected "
 			"for bitwise operation");
 	return NULL;
     }
-    switch (op_type) {
-    case AND:
+    
+    setunused(s);
+    setunused(o);
+    
+    if (strcmp(operation, "and") == 0) {
 	for (i = 0; i < self->ob_size; i++)
 	    s->ob_item[i] &= o->ob_item[i];
-	break;
-    case OR:
+    }
+    else if (strcmp(operation, "or") == 0) {
 	for (i = 0; i < self->ob_size; i++)
 	    s->ob_item[i] |= o->ob_item[i];
-	break;
-    case XOR:
+    }
+    else if (strcmp(operation, "xor") == 0) {
 	for (i = 0; i < self->ob_size; i++)
 	    s->ob_item[i] ^= o->ob_item[i];
-	break;
     }
+    else {
+	PyErr_SetString(PyExc_ValueError, "bitwise operation unknown");
+	return NULL;
+    }
+    
     Py_RETURN_NONE;
 }
 
@@ -1068,7 +1059,7 @@ PyDoc_STRVAR(bitwise_doc,
 \n\
 Perform in-place bitwise operations, operations is a string specifying the\n\
 operation ('and', 'or', 'xor').  This method is not meant to be called\n\
-directly, but from, it is called by __and__, __iadd__, etc....");
+directly, but from, it is called by __and__, __iadd__, etc...");
 
 
 static PyObject *
@@ -1637,38 +1628,29 @@ bitarray_delitem(bitarrayobject *self, PyObject *a)
 static int
 check_CodeDict(PyObject *dict)
 {
-    PyObject *values = NULL, *bits;
-    Py_ssize_t N, i;
-    int ret = -1;
+    PyObject *key, *value;
+    Py_ssize_t i = 0;
     
     if (!PyDict_Check(dict)) {
         PyErr_SetString(PyExc_TypeError, "dictionary expected");
-        goto error;
+        return -1;
     }
-    N = PyDict_Size(dict);
-    if (N == 0) {
-        PyErr_SetString(PyExc_ValueError, "dict is empty");
-        goto error;
+    if (PyDict_Size(dict) == 0) {
+        PyErr_SetString(PyExc_ValueError, "prefix code empty");
+        return -1;
     }
-    values = PyDict_Values(dict);
-    if (values == NULL || PyList_Size(values) != N)
-        goto error;
-    
-    for (i = 0; i < N; i++) {
-        bits = PyList_GetItem(values, i);
-        if (bits == NULL)
-            goto error;
-        
-        if (!bitarray_Check(bits)) {
+    while (PyDict_Next(dict, &i, &key, &value)) {
+        if (!bitarray_Check(value)) {
             PyErr_SetString(PyExc_TypeError,
                             "bitarray expected for dictionary value");
-            goto error;
+            return -1;
         }
+	if (((bitarrayobject *) value)->nbits == 0) {
+	    PyErr_SetString(PyExc_ValueError, "non-empty bitarray expected");
+            return -1;
+	}
     }
-    ret = 0; /* success */
- error:
-    Py_XDECREF(values);
-    return ret;
+    return 0;
 }
 
 static PyObject *
@@ -1690,12 +1672,17 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
     /* Extend self with the bitarrays from codedict */
     while ((symbol = PyIter_Next(iter)) != NULL) {
         bits = PyDict_GetItem(codedict, symbol);
-        Py_DECREF(symbol);
         if (bits == NULL) {
-            PyErr_SetString(PyExc_ValueError, "symbol not found in code");    
-            Py_DECREF(iter);
+	    char buff[256];
+	    
+	    sprintf(buff, "no symbol %s in prefix code",
+		    PyString_AsString(PyObject_Repr(symbol)));
+	    PyErr_SetString(PyExc_ValueError, strdup(buff));
+	    Py_DECREF(symbol);
+	    Py_DECREF(iter);
             return NULL;
         }
+        Py_DECREF(symbol);
         bitarray_extend(self, bits);
     }
     Py_DECREF(iter);
@@ -1708,50 +1695,137 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
 PyDoc_STRVAR(encode_doc,
 "encode(code, iterable)\n\
 \n\
-Given a code (a dict mapping symbols to bitarrays),\n\
+Given a prefix code (a dict mapping symbols to bitarrays),\n\
 iterates over iterable object with symbols, and extends the bitarray\n\
-with the corresponding bitarray for each symbols, using the code.");
+with the corresponding bitarray for each symbols.");
 
 
+/* Return the leave node resulting from traversing the binary tree,
+   or, when the iteration is finished, NULL
+*/
+static PyObject *
+btree_traverse(PyObject *iter, PyObject *tree)
+{
+    PyObject *subtree, *v;
+    long vi;
+    
+    v = PyIter_Next(iter);
+    if (v == NULL) /* stop iterator */
+	return NULL;
+    
+    vi = PyObject_IsTrue(v);
+    Py_DECREF(v);
+    
+    subtree = PyList_GetItem(tree, vi);
+    
+    if (PyList_Check(subtree) && PyList_Size(subtree) == 2)
+	return btree_traverse(iter, subtree);
+    else
+	return subtree;
+}
+
+/* Return new empty btree, i.e. [[], []].
+   The empty list is our placeholder for undefined.
+   The reason for choosing this placeholder is that lists are unhashable,
+   and can therefore not be symbols in the dict keys.
+*/
+static PyObject *
+btree_new()
+{
+    PyObject *list;
+    
+    list = PyList_New(2);
+    PyList_SetItem(list, 0, PyList_New(0));
+    PyList_SetItem(list, 1, PyList_New(0));
+    return list;
+}
+
+#define IS_EMPTY_LIST(x)  (PyList_Check(x) && PyList_Size(x) == 0)
+
+/* Insert symbol which is mapped to bitarray into binary tree */
+static int
+btree_insert(PyObject *tree, PyObject *symbol, PyObject *b)
+{
+    PyObject *a;
+    bitarrayobject *bb;
+    int ret, vi;
+    
+    bb = (bitarrayobject *) b;
+    vi = getbit(bb, 0);
+    if (bb->nbits > 1) {
+	/* create new subtree if necessary */
+	if (IS_EMPTY_LIST(PyList_GetItem(tree, vi)))
+	    PyList_SetItem(tree, vi, btree_new());
+	
+	/* a = b[1:], and insert symbol */
+	a = bitarray_copy(bb);
+	delete((bitarrayobject *) a, 0, 1);
+	ret = btree_insert(PyList_GetItem(tree, vi), symbol, a);
+	Py_INCREF(symbol);
+	return ret;
+    }
+    
+    /* we have reached the leaf node where the symbol lives */
+    if (!IS_EMPTY_LIST(PyList_GetItem(tree, vi))) {
+	PyErr_SetString(PyExc_ValueError, "prefix code ambiguous");
+	return -1;
+    }
+    PyList_SetItem(tree, vi, symbol);
+    Py_INCREF(symbol);
+    return 0;
+}
+
 static PyObject *
 bitarray_decode(bitarrayobject *self, PyObject *codedict)
 {
-    PyObject *res, *item, *d, *b;
-    idx_t pos = 0;
-    int i, size, minsize = 1, maxsize = 100;
-
-    if (check_CodeDict(codedict) < 0)
-        return NULL;
-
-    res = PyList_New(0);        /* list of symbols to be returned */
+    PyObject *tree = NULL, *iter = NULL, *res = NULL, *symbol, *ba;
+    Py_ssize_t i = 0;
     
-    // XXX
-    while (pos < self->nbits) {
-        for (size = minsize; size < maxsize; size++) {
-            //printf("size = %d\n", size);
-            b = newbitarrayobject(self->ob_type, size, self->endian);
-            for (i = 0; i < size; i++)
-                setbit((bitarrayobject *) b, i,
-                       getbit(self, i + pos));
-            
-            item = PyDict_GetItem(d, b);
-            dealloc((bitarrayobject *) b);
-            if (item != NULL) {
-                PyList_Append(res, item);
-                pos += size;
-                break;
-            }
-        }
-        if (size > 95)
-            break;
+    if (check_CodeDict(codedict) < 0)
+        goto finish;
+    
+    /* create binary tree (with symbols at leave nodes) for traversing */
+    tree = btree_new();
+    while (PyDict_Next(codedict, &i, &symbol, &ba))
+	if (btree_insert(tree, symbol, ba) < 0)
+	    goto finish;
+    
+    /* XXX printf("tree = %s\n", PyString_AsString(PyObject_Repr(tree))); */
+    
+    iter = PyObject_GetIter((PyObject *) self);
+    if (iter == NULL)
+	goto finish;
+    
+    /* traverse binary tree and append symbols to the result list */
+    res = PyList_New(0);
+    while (1) {
+	symbol = btree_traverse(iter, tree);
+	if (symbol == NULL)
+	    break;
+	
+	if (IS_EMPTY_LIST(symbol)) {
+	    PyErr_SetString(PyExc_ValueError,
+			    "prefix code does not match data in bitarray");
+	    Py_DECREF(res);
+	    res = NULL;
+	    /* XXX res sould contain symbols read so far, but then
+	       res is not NULL an no exception is raised.
+	    */
+	    goto finish;
+	}
+	PyList_Append(res, symbol);
+	Py_INCREF(symbol);
     }
+finish:
+    Py_XDECREF(tree);
+    Py_XDECREF(iter);
     return res;
 }
 
 PyDoc_STRVAR(decode_doc,
 "decode(code)\n\
 \n\
-Given a code (a dict mapping symbols to bitarrays),\n\
+Given a prefix code (a dict mapping symbols to bitarrays),\n\
 decode the content of the bitarray and return the list of symbols.");
 
 
@@ -1783,9 +1857,9 @@ bitarray_methods[] = {
      copy_doc},
     {"__delitem__",  (PyCFunction)bitarray_delitem,     METH_O,
      0},
-    {"_decode",       (PyCFunction)bitarray_decode,     METH_O,
+    {"decode",       (PyCFunction)bitarray_decode,      METH_O,
      decode_doc},
-    {"_encode",       (PyCFunction)bitarray_encode,     METH_VARARGS,
+    {"encode",       (PyCFunction)bitarray_encode,      METH_VARARGS,
      encode_doc},
     {"endian",       (PyCFunction)bitarray_endian,      METH_NOARGS,
      endian_doc},
@@ -1836,7 +1910,7 @@ bitarray_methods[] = {
     {"unpack",       (PyCFunction)bitarray_unpack,      METH_VARARGS |
                                                         METH_KEYWORDS,
      unpack_doc},
-    {NULL,      NULL}       /* sentinel */
+    {NULL, NULL, 0, NULL}  /* sentinel */
 };
 
 
@@ -2010,11 +2084,12 @@ static PyObject *bitarray_iter(bitarrayobject *self);
 
 static PyTypeObject Bitarraytype = {
     PyObject_HEAD_INIT(NULL)
-    0,
-    "_bitarray._bitarray",
-    sizeof(bitarrayobject),
-    0,
-    (destructor) dealloc,                     /* tp_dealloc */
+    0,                                        /* ob_size */
+    "_bitarray._bitarray",                    /* tp_name */
+    sizeof(bitarrayobject),                   /* tp_basicsize */
+    0,                                        /* tp_itemsize */
+    /* methods */
+    (destructor) bitarray_dealloc,            /* tp_dealloc */
     0,                                        /* tp_print */
     0,                                        /* tp_getattr */
     0,                                        /* tp_setattr */
@@ -2053,12 +2128,12 @@ static PyTypeObject Bitarraytype = {
 };
 
 
-/*********************** Bitarray Iterator **************************/
+/************************** Bitarray Iterator **************************/
 
 typedef struct {
     PyObject_HEAD
-    idx_t           index;
-    bitarrayobject*     ao;
+    idx_t index;
+    bitarrayobject* bao;
 } bitarrayiterobject;
 
 static PyTypeObject BitarrayIter_Type;
@@ -2079,7 +2154,7 @@ bitarray_iter(bitarrayobject *self)
         return NULL;
     
     Py_INCREF(self);
-    it->ao = self;
+    it->bao = self;
     it->index = 0;
     PyObject_GC_Track(it);
     return (PyObject *) it;
@@ -2089,8 +2164,8 @@ static PyObject *
 bitarrayiter_next(bitarrayiterobject *it)
 {
     assert (BitarrayIter_Check(it));
-    if (it->index < it->ao->nbits)
-        return PyBool_FromLong(getbit(it->ao, it->index++));
+    if (it->index < it->bao->nbits)
+        return PyBool_FromLong(getbit(it->bao, it->index++));
     
     return NULL;
 }
@@ -2099,14 +2174,14 @@ static void
 bitarrayiter_dealloc(bitarrayiterobject *it)
 {
     PyObject_GC_UnTrack(it);
-    Py_XDECREF(it->ao);
+    Py_XDECREF(it->bao);
     PyObject_GC_Del(it);
 }
 
 static int
 bitarrayiter_traverse(bitarrayiterobject *it, visitproc visit, void *arg)
 {
-    Py_VISIT(it->ao);
+    Py_VISIT(it->bao);
     return 0;
 }
 
@@ -2170,7 +2245,7 @@ Return the number of bytes necessary to store n bits.");
 
 static PyMethodDef module_functions[] = {
     {"bits2bytes",   bits2bytes,   METH_O,  bits2bytes_doc},
-    {NULL, NULL, 0, NULL}        /* Sentinel */
+    {NULL, NULL, 0, NULL}  /* sentinel */
 };
 
 /*********************** Install Module **************************/
