@@ -302,10 +302,10 @@ class MetaDataTests(unittest.TestCase):
         self.assertEqual(len(bi), 5)
         
         self.assert_(isinstance(bi[0], (int, long)))
-        self.assert_(isinstance(bi[1], long))
+        self.assert_(isinstance(bi[1], (int, long)))
         self.assert_(isinstance(bi[2], str))
         self.assert_(isinstance(bi[3], int))
-        self.assert_(isinstance(bi[4], long))
+        self.assert_(isinstance(bi[4], (int, long)))
         
         for n in xrange(50):
             bi = bitarray(n).buffer_info()
@@ -499,14 +499,19 @@ class MiscTests(unittest.TestCase, Util):
                 acc.append(b)
             self.assertEqual(acc, lst)
 
-    def test_asDictKey(self):
-        d = {bitarray('011') : 3,
-             bitarray('101') : 5,
-             bitarray('001') : 1,
-             bitarray('000') : 0,
-             bitarray('1111') : 15}
-
-        self.assertEqual(d[bitarray('000')], 0)
+    def test_iterable(self):
+        a = iter(bitarray('011'))
+        self.assertEqual(a.next(), False)
+        self.assertEqual(a.next(), True)
+        self.assertEqual(a.next(), True)
+        self.assertRaises(StopIteration, a.next)
+        
+    def test_assignment(self):
+        a = bitarray('00110111001')
+        a[1:3] = a[7:9]
+        a[-1:] = a[:1]
+        b = bitarray('01010111000')
+        self.assertEqual(a, b)
 
     def test_compare(self):
         for a in self.randombitarrays():
@@ -1523,6 +1528,167 @@ tests.append(FileTests)
 
 # ---------------------------------------------------------------------------
 
+class PrefixCodeTests(unittest.TestCase):
+
+    def test_encode_check_codedict(self):
+        a = bitarray()
+        self.assertRaises(TypeError, a.encode, 0, '')
+        self.assertRaises(ValueError, a.encode, {}, '')
+        self.assertRaises(TypeError, a.encode, {'a':42}, '')
+        self.assertRaises(ValueError, a.encode, {'a':bitarray()}, '')
+        # 42 not iterable
+        self.assertRaises(TypeError, a.encode, {'a':bitarray('0')}, 42)
+        self.assertEqual(len(a), 0)
+        
+    def test_encode_string(self):
+        a = bitarray()
+        d = {'a':bitarray('0')}
+        a.encode(d, '')
+        self.assertEqual(a, bitarray())
+        a.encode(d, 'a')
+        self.assertEqual(a, bitarray('0'))
+        self.assertEqual(d, {'a':bitarray('0')})
+        
+    def test_encode_list(self):
+        a = bitarray()
+        d = {'a':bitarray('0')}
+        a.encode(d, [])
+        self.assertEqual(a, bitarray())
+        a.encode(d, ['a'])
+        self.assertEqual(a, bitarray('0'))
+        self.assertEqual(d, {'a':bitarray('0')})
+        
+    def test_encode_iter(self):
+        a = bitarray()
+        d = {'a':bitarray('0'), 'b':bitarray('1')}
+        a.encode(d, iter('abba'))
+        self.assertEqual(a, bitarray('0110'))
+        
+        def foo():
+            for c in 'bbaabb':
+                yield c
+                
+        a.encode(d, foo())
+        self.assertEqual(a, bitarray('0110110011'))
+        self.assertEqual(d, {'a':bitarray('0'), 'b':bitarray('1')})
+
+    def test_encode(self):
+        d = {'I':bitarray('1'),
+             'l':bitarray('01'),
+             'a':bitarray('001'),
+             'n':bitarray('000')}
+        a = bitarray()
+        a.encode(d, 'Ilan')
+        self.assertEqual(a, bitarray('101001000'))
+        a.encode(d, 'a')
+        self.assertEqual(a, bitarray('101001000001'))
+        self.assertEqual(d, {'I':bitarray('1'), 'l':bitarray('01'),
+                             'a':bitarray('001'), 'n':bitarray('000')})
+        self.assertRaises(ValueError, a.encode, d, 'arvin')
+
+
+    def test_decode_check_codedict(self):
+        a = bitarray()
+        self.assertRaises(TypeError, a.decode, 0)
+        self.assertRaises(ValueError, a.decode, {})
+        # 42 not iterable
+        self.assertRaises(TypeError, a.decode, {'a':42})
+        self.assertRaises(ValueError, a.decode, {'a':bitarray()})
+
+    def test_decode_simple(self):
+        d = {'I':bitarray('1'),
+             'l':bitarray('01'),
+             'a':bitarray('001'),
+             'n':bitarray('000')}
+        a = bitarray('101001000')
+        self.assertEqual(a.decode(d), ['I', 'l', 'a', 'n'])
+        self.assertEqual(d, {'I':bitarray('1'), 'l':bitarray('01'),
+                             'a':bitarray('001'), 'n':bitarray('000')})
+        self.assertEqual(a, bitarray('101001000'))
+
+    def test_decode_empty(self):
+        d = {'a':bitarray('1')}
+        a = bitarray()
+        self.assertEqual(a.decode(d), [])
+        self.assertEqual(d, {'a':bitarray('1')})
+        
+    def test_decode_buggybitarray(self):
+        d = {'a':bitarray('0')}
+        a = bitarray('1')
+        self.assertRaises(ValueError, a.decode, d)
+        self.assertEqual(a, bitarray('1'))
+        self.assertEqual(d, {'a':bitarray('0')})
+
+    def test_decode_buggybitarray2(self):
+        d = {'a':bitarray('00'), 'b':bitarray('01')}
+        a = bitarray('1')
+        self.assertRaises(ValueError, a.decode, d)
+        self.assertEqual(a, bitarray('1'))
+
+    def test_decode_ambiguous_code(self):
+        d = {'a':bitarray('0'), 'b':bitarray('0'), 'c':bitarray('1')}
+        a = bitarray()
+        self.assertRaises(ValueError, a.decode, d)
+
+    def test_decode_ambiguous2(self):
+        d = {'a':bitarray('01'), 'b':bitarray('01'), 'c':bitarray('1')}
+        a = bitarray()
+        self.assertRaises(ValueError, a.decode, d)
+
+
+    def test_miscitems(self):
+        d = {None :bitarray('00'),
+             0    :bitarray('110'),
+             1    :bitarray('111'),
+             ''   :bitarray('010'),
+             2    :bitarray('011')}
+        a = bitarray()
+        a.encode(d, [None, 0, 1, '', 2])
+        self.assertEqual(a, bitarray('00110111010011'))
+        self.assertEqual(a.decode(d), [None, 0, 1, '', 2])
+
+    def test_real_example(self):
+        code = {' '  : bitarray('001'),
+                '.'  : bitarray('0101010'),
+                'a'  : bitarray('0110'),
+                'b'  : bitarray('0001100'),
+                'c'  : bitarray('000011'),
+                'd'  : bitarray('01011'),
+                'e'  : bitarray('111'),
+                'f'  : bitarray('010100'),
+                'g'  : bitarray('101000'),
+                'h'  : bitarray('00000'),
+                'i'  : bitarray('1011'),
+                'j'  : bitarray('0111101111'),
+                'k'  : bitarray('00011010'),
+                'l'  : bitarray('01110'),
+                'm'  : bitarray('000111'),
+                'n'  : bitarray('1001'),
+                'o'  : bitarray('1000'),
+                'p'  : bitarray('101001'),
+                'q'  : bitarray('00001001101'),
+                'r'  : bitarray('1101'),
+                's'  : bitarray('1100'),
+                't'  : bitarray('0100'),
+                'u'  : bitarray('000100'),
+                'v'  : bitarray('0111100'),
+                'w'  : bitarray('011111'),
+                'x'  : bitarray('0000100011'),
+                'y'  : bitarray('101010'),
+                'z'  : bitarray('00011011110')}
+        a = bitarray()
+        a.encode(code, 'the quick brown fox jumps over the lazy dog.')
+        self.assertEqual(a, bitarray('01000000011100100001001101000100101100'
+          '00110001101000100011001101100001111110010010101001000000010001100'
+          '10111101111000100000111101001110000110000111100111110100101000000'
+          '0111001011100110000110111101010100010101110001010000101010'))
+        self.assertEqual(''.join(a.decode(code)),
+                         'the quick brown fox jumps over the lazy dog.')
+
+tests.append(PrefixCodeTests)
+
+# ---------------------------------------------------------------------------
+
 def pages():
     dat = open('/proc/%i/statm' % os.getpid()).read()
     return int(dat.split()[0])
